:- module( svg, [
                    svg/2,
                    svg_change_dim/4,
                    svg_legend/1,
                    svg_size/3,
                    svg_tag_coords/3, svg_tag_coords/4,
                    svg_version/2
    ] ).

:- use_module( library(lib) ).
:- lib(source(svg),homonyms(true)).

:- lib(os_lib).
:- lib(debug_call).
:- lib(pack_errors).

% :- lib(stoics_lib:at_con/3).

:- lib(svg_size/3).
:- lib(svg_change_dim/3).
:- lib(svg_tag_coords/3).
:- lib(svg_legend/1).

:- lib(end(svg)).

/** <module> Manipulating SVG xmls.

Read/write SVGs as xmls.

---++ Pack info 

@author   nicos angelopoulos
@version    0.1 2021/1/19
@see http://stoics.org.uk/~nicos/sware/svg
@see http://stoics.org.uk/~nicos/sware/svg/doc/html/svg.html
@see pack(svg/examples/)
@see pack(svg/scripts/)

*/

/** svg( +SvgF, -Svg ).
    svg( +SvgF, +Svg ).
    svg( -SvgF, +Svg ).

Load and write Svg files (SvgF) to and from terms (Svg).<br>
An unbound Svg triggers loading, else terms are written to file.<br>
An unbound SvgF passes Svg into SvgF.

==
?- svg( pack('svg/examples/rea_smp-e10_fclr.svg'), Svg ).
Svg = svg([width='497pt', height='404pt', viewBox='0.00 0.0...]....)

==

@author nicos angelopoulos
@version  0.1 2021/1/19

*/

svg( SvgOut, Svg ) :-
    var( SvgOut ),
    !,
    Svg = SvgOut.
svg( svg(Atts,Elems), Svg ) :-
    !,
    Svg = svg(Atts,Elems ).
svg( SvgF, Svg ) :-
    var( Svg ),
    !,
    absolute_file_name( SvgF, AbsF, [expand(true),solutions(first),file_errors(error),access(write)] ),
    load_xml( AbsF, XmlSvg, [] ),
    XmlSvg = [element(svg,Atts,Elems)],
    Svg = svg(Atts,Elems). % fixme: or is it better to leave it as XML element ? or it better to seek the only element(g,_,_) ??
svg( SvgF, Svg ) :-
    absolute_file_name( SvgF, AbsF, [expand(true),solutions(first),file_errors(error),access(write)] ),
    Svg = svg(Atts,Elems),
    Xml = [element(svg,Atts,Elems)],
    open( AbsF, write, SvgO ),
    % xml_write( SvgO, Xml, [dtd('http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'),layout(false)] ).
    % xml_write( SvgO, Xml, [dtd('/tmp/svg11.dtd'),layout(false)] ).
    % Type = svg(abc),
    % xml_write( SvgO, Xml, [layout(false),doctype(Type)] ),
    % fixme: add doctype
    xml_write( SvgO, Xml, [layout(false)] ),
    nl( SvgO ),
    close( SvgO ).


/** svg_version( -Version, -Date ).

Current version and release date for the library.

==
?- svg_version( V, D ).
V = 0:1:0,
D = date(2021, 2, 19).
==
*/
svg_version( 0:1:0, date(2021,1,19) ).
